.TH sam9407 4 "20 Mar 2000" Linux "Linux Programmer's Manual"
.SH NAME
sam9407 \- sam9407 device driver
.SH DESCRIPTION
\fBsam9407\fR is a device driver for sam9407 based soundcards.

.SH SYNOPSIS
.TP
.B "insmod sam9407.o \fR[\fBsamMajor=\fI#\fR] [\fBsamCardsMap=\fI#,#...\fR]"

.TP
.B "#include <sys/sam9407.h>"

.SH "SUPPORTED CARDS"
.IP * 2
Hoontech ST128 Gold & Ruby
.IP *
Hoontech Soundtrack Digital Audio (partly)
.IP *
Guillemot Maxi Sound Home Studio Pro 64, Home Studio 2, Dynamic 3D
.IP *
Terratec EWS 64 S/L/XL

.SH "PARAMETERS"

.TP
.B "samMajor=\fI#"
Specify the device major number that shall be used. Normally the user
should leave that option untouched, so the device driver will use
the official sam9407 device major number.
However, if you want to access sam9407 using the OSS devices nodes,
device number \fB14\fR might be used here.

.TP
.B "samCardsMap=\fI#,#..."
On startup, sam9407 will first search the ISA bus for supported
sam9407 soundcards, followed by the PCI bus.
If you want a different order of sam9407 devices, pass the desired
position numbers along with this option.

.SH "IOCTL(/dev/sam9407/card\fR[0-9]\fB/mixer\fR,\fB /dev/sam\fR[0-9]\fB_mixer)"

.TP
.B "SAM_IOC_API_INFO (IOR)"
Return the API version number of the installed driver and the
API class of the installed firmware as:
.sp
.nf
typedef struct {
  unsigned version;
  SamApiClass apiClass;
} SamApiInfo;
.fi
.sp
.RS +5
.TP
.B "version"
The API version of the installed driver. You can assert
equality with the constant \fBSAM_MIXER_API_VERSION\fR to ensure
that the application was compiled with the matching headerfile.
.TP
.B "apiClass"
Defines the set of control id's you need to talk with
the installed firmware by means of
\fBSAM_IOC_AUDIO_\fR{\fBGET\fR,\fBSET\fR,\fBINFO\fR} and 
\fBSAM_IOC_MIXER_\fR{\fBGET\fR,\fBSET\fR,\fBINFO\fR}.
Possible values are:
.RS +5
.TP
.B "SAM_API_CLASS_NONE"
No firmware has been loaded so far.
.TP
.B "SAM_API_CLASS_VANILLA"
The standard API, which uses the controls listed in
\fBSamAudioId\fR and \fBSamMixerId\fR.
Please refer to the firmware documentation for a description
of all those parameters.
.TP
.B "SAM_API_CLASS_HOMSTDR"
The Home Studio Recording API, which uses the controls
listed in \fBSamAudioHomstdrId\fR and \fBSamMixerHomstdrId\fR.
Please refer to the firmware documentation for a description
of all those parameters.
.RS -5
.RS -5

.TP
.B "SAM_IOC_SYNC_JOIN (IOWR)"
Join or create a sync playback group. A handle of type int is being
passed as parameter. If this handle is negative, a new playback group
will be created and returned in the handle parameter.
Otherwise the given playback group will be joined.
If the given playback group is already playing, EBUSY will be returned.
Please note that unlike most other devices, the joined playback group subscribed
through the mixer interface will never start playing, unless the mixer unsubscribes again
using \fBSAM_IOC_SYNC_LEAVE\fR. The only use of sync groups together with the mixer
interface is to cue them in.

.TP
.B "SAM_IOC_SYNC_LEAVE (IO)"
Leave the currently joined sync playback group again.

.TP
.B "SAM_IOC_SYNC_INFO (IOR)"
Return the currently joined sync playback group in the handle parameter
of type int. If no playback group has been joined, the returned handle
will be negative.

.TP
.B "SAM_IOC_DRIVER_INFO (IOR)"
Return information about the hardware and firmware located at device as:
.sp
.nf
typedef struct {
  SamDriverVersion driverVersion;
  char hardware[16];
  char firmware[16];
  unsigned char haveAudio, haveMidi, haveMod;
  unsigned memTotal, memFree, memSBanks, memMod, memAudio;
  unsigned playChannelsUsed, playChannelsTotal;
  unsigned recordChannelsUsed, recordChannelsTotal;
  unsigned peakMeterValues;
} SamDriverInfo;
.fi
.sp
.RS +5
.TP
.B "driverVersion"
Contains the driver version information as being distributed.
The three parts of the version information are contained in:
.RS +5
.sp
.nf
typedef struct {
  unsigned char major, minor, tiny;
} SamDriverVersion;
.fi
.sp
.RS -5
.TP
.B "hardware"
The abbreviated name of the card.
.TP
.B "firmware"
The abbreviated name of the firmware. If the string is empty,
no firmware has been loaded so far.
.TP
.B "haveAudio"
Contains a nonzero value, if the firmware supports audio streaming.
.TP
.B "haveMidi"
Contains a nonzero value, if the firmware supports midi and synthesizer.
.TP
.B "haveMod"
Contains a nonzero value, if the firmware supports static wave output
(mod device).
.TP
.B "memTotal"
The total amount of RAM available on the card in words (2 bytes).
.TP
.B "memFree"
The amount of free memory available on the card in words (2 bytes).
.TP
.B "memSBanks"
The amount of memory used by soundbanks in words (2 bytes).
.TP
.B "memMod"
The amount of memory used by static wave samples (mod device) in words (2 bytes).
.TP
.B "memAudio"
The amount of memory used by streaming audio in words (2 bytes).
.TP
.B "playChannelsUsed"
How many playback channels are currently used.
.TP
.B "playChannelsTotal"
The total amount of playback channels available on this
firmware.
.TP
.B "recordChannelsUsed"
How many recording channels are currently used.
.TP
.B "recordChannelsTotal"
How many recording channels this firmware offers.
.TP
.B "peakMeterValues"
The number of peak meter entries this firmware does support.
.RS -5

.TP
.B "SAM_IOC_HW_HINTS_GET (IOR)"
Return the hardware hints that have been set previously by a call
to \fBSAM_IOC_HW_HINTS_SET\fR. Please refer to \fBSAM_IOC_HW_HINTS_SET\fR
for a description of the parameter.

.TP
.B "SAM_IOC_HW_HINTS_SET (IOR)"
Gives some hardware hints to the driver which are defined as:
.sp
.nf
typedef struct {
  unsigned cardMem;
} SamHardwareHints;
.fi
.sp
.RS +5
.TP
.B "cardMem"
The amount of memory installed on the card in words (2 bytes).
\fBcardMem\fR is only used if the uploaded firmware cannot detect
the amount of memory installed, in which case an autprobe algorithm
would try to detect the amount of memory on the card. If this hint
is set however, no autoprobing will take place.
.PP
Using this function will require root priviledge.
.RS -5

.TP
.B "SAM_IOC_FIRMWARE_LOAD (IOW)"
Upload the firmware defined in:
.sp
.nf
typedef struct {
  unsigned short *data;
  unsigned size;
} SamFirmware;
.fi
.sp
.RS +5
.TP
.B "data"
points to the memory location where the firmware is stored.
.TP
.B "size"
defines the number of words (2 bytes) stored at \fBdata\fR.
.PP
Depending on how the module has been compiled, using this function
will require root priviledge.
This operation will return EBUSY, in case some process is still using
the soundcard.
.RS -5

.TP
.B "SAM_IOC_MIXER_GET (IOWR)"
Retrieve mixer control values, that will be stored in the vector passed as:
.sp
.nf
typedef struct SamControl {
  unsigned char id, addr;
  char isConfigured;
  unsigned value;
} SamControl;
.fi
.sp
.RS +5
.TP
.B "id"
Defines the control id of the value you want to retrieve.
Valid control id's are firmware independent hardware controls listed in
\fBSamMixerHwId\fR and those belonging to the \fBapiClass\fR returned by
\fBSAM_IOC_API_INFO\fR.
The last element in the vector needs to have an \fBid\fR field
with the value \fBSAM_MIXER_ID_SENTINEL\fR to mark the end of
the vector.
.TP
.B "addr"
Some mixer controls need an additional address tag. Please refer
to firmware documentation for a description
of those controls.
.TP
.B "isConfigured"
Is unused in mixer controls, and will always be non-zero for
\fBSAM_IOC_MIXER_GET\fR.
.TP
.B "value"
Will contain the value for this particular mixer setting, but only
if the requested mixer control is supported by the device.
You can query the list of supported mixer controls by using
\fBSAM_IOC_MIXER_INFO\fR.
.RS -5

.TP
.B "SAM_IOC_MIXER_SET (IOW)"
Set mixer control values. The parameter being passed is described
in \fBSAM_IOC_MIXER_GET\fR.
Each \fBvalue\fR for the mixer controls being supported will be
sent to the firmware and/or hardware.
Some mixer controls, that would reset the firmware, can't be set while
the device is currently used. The ioctl will return EBUSY in those cases.
Many mixer settings will be lost after issuing a reset control change.
Refer to the SAM9407 firmware manual for further details.

.TP
.B "SAM_IOC_MIXER_INFO (IOWR)"
Query supported mixer types. The parameter being passed is described
in \fBSAM_IOC_MIXER_GET\fR. In case the given mixer control is
supported by the firmware and/or hardware, the \fBvalue\fR field will
be set to a non-zero value.

.TP
.B "SAM_IOC_BANK_LOAD (IOW)"
Upload the soundbank defined in:
.sp
.nf
typedef struct {
  unsigned char id;
  char *data;
  unsigned size;
} SamSoundBank;
.fi
.sp
.RS +5
.TP
.B "id"
Defines the slot where the soundbank should be placed. There's only support
for up to \fBSAM_NROF_SBANKS\fR soundbanks all together, which start at
number zero.
.TP
.B "data"
points to the memory location where the soundbank is stored
as \fB.94B\fR format.
.TP
.B "size"
defines the number of words (2 bytes) stored at \fBdata\fR.
.PP
If a soundbank was already loaded at slot number \fBid\fR, it will be discarded first.
.RS -5

.TP
.B "SAM_IOC_BANK_UNLOAD (IOW)"
Unload a soundbank from slot number
.sp
.nf
unsigned char
.fi
.sp

.TP
.B "SAM_IOC_BANK_PRIO_CHG (IOW)"
Change the priority of a soundbank given in:
.sp
.nf
typedef struct {
  unsigned char id;
  unsigned short priority;
} SamSoundBankPrio;
.fi
.sp
.RS +5
.TP
.B "id"
Defines the slot number of the soundbank beinng subject to change.
.TP
.B "priority"
Defines the new priority of the soundbank. The higher the number,
the higher the priority level is.
.RS -5

.TP
.B "SAM_IOC_MIXER_AUDIO_GET (IOWR)"
.TP
.B "SAM_IOC_MIXER_AUDIO_SET (IOW)"
.TP
.B "SAM_IOC_MIXER_AUDIO_INFO (IOWR)"
These ioctl's correspond to the matching
\fBSAM_IOC_AUDIO_\fR{\fBGET\fR,\fBSET\fR,\fBINFO\fR}
call, with the difference that you need to provide a channel number
for the target audio channel in the parameter defined as:
.sp
.nf
typedef struct {
  unsigned char channel;
  SamControl *controls;
} SamRedirectedAudioControl;
.fi
.sp
.RS +5
.TP
.B "channel"
The audio channel number, as being returned by a \fBSAM_IOC_CHANNEL_INFO\fR ioctl.
.TP
.B "controls"
A pointer to the controls that are passed to the
\fBSAM_IOC_AUDIO_\fR{\fBGET\fR,\fBSET\fR,\fBINFO\fR} call.
.RS -5

.TP
.B "SAM_IOC_CHANNEL_OWNER_INFO (IOWR)"
Retrieve and store the process id's of the owning processes for all known channels,
in a structure defined as:
.sp
.nf
typedef struct {
  int *pids;
  unsigned size;
} SamChannelOwnerInfo;
.fi
.sp
.RS +5
.TP
.B "pids"
Points to a vector where all the process id's will be stored.
If the value is zero, the corresponding audio channel is currently not in use.
The length of this vector needs to be defined in \fBsize\fR.
.TP
.B "size"
The number of process id's the vector pointed to by \fBpids\fR has space
for. After a successful call, \fBsize\fR will contain the number of
process id's stored at \fBpids\fR.
.RS -5

.TP
.B "SAM_IOC_BANK_INFO (IOR)"
Query all soundbanks and return information in a vector of size
\fBSAM_NROF_SBANKS\fR with the type:
.sp
.nf
typedef struct {
  char name[SAM_SBANK_NAME_LEN+1];
  unsigned short priority;
} SamSoundBankInfo;
.fi
.sp
.RS +5
.TP
.B "name"
The name of the soundbank, being null terminated.
.TP
.B "priority"
The current priority level of the soundbank.
.RS -5

.TP
.B "SAM_IOC_PEAK_METER_INFO (IOWR)"
Retrieve the peak meter table, which is offered
by some firmare's only. You can obtain the number of peak meter
entries supported by the firmware by doing a \fBSAM_IOC_DRIVER_INFO\fR.
The peak meter values will be stored in:
.sp
.nf
typedef struct {
  unsigned short *data;
  unsigned size;
} SamPeakMeterInfo;
.fi
.sp
.RS +5
.TP
.B "data"
Points to a vector where all the peak meter values will be stored.
The length of this vector needs to be defined in \fBsize\fR.
.TP
.B "size"
The number of peak meter entries the vector pointed to by \fBdata\fR has space
for. After a successful call, \fBsize\fR will contain the number peak
meter values stored at \fBdata\fR.
.RS -5

.SH "IOCTL(/dev/sam9407/card\fR[0-9]\fB/{dsp,audio}\fR,\fB /dev/sam\fR[0-9]\fB_{dsp,audio})"
.TP
.B "SAM_IOC_API_INFO (IOR)"
Refer to the \fBIOCTL(/dev/sam9407/card\fR[0-9]\fB/mixer\fR,\fB /dev/sam\fR[0-9]\fB_mixer)\fR
section for a description. Please note that on this device
the API version number returned shall be compared with the
constant \fBSAM_AUDIO_API_VERSION\fR.
.TP
.B "SAM_IOC_SYNC_JOIN (IOWR)"
.TP
.B "SAM_IOC_SYNC_LEAVE (IO)"
.TP
.B "SAM_IOC_SYNC_INFO (IOR)"
Refer to the \fBIOCTL(/dev/sam9407/card\fR[0-9]\fB/mixer\fR,\fB /dev/sam\fR[0-9]\fB_mixer)\fR
section for a description.
.TP
.B "SAM_IOC_CHANNEL_INFO (IOR)"
Retrieve the channel id numbers for both play and record direction. These
numbers are used in event reporting as well as with redirected audio controls
on the mixer device. The return values will be stored in:
.sp
.nf
typedef struct {
  char play, record;
} SamChannelNumbers;
.fi
.sp
.RS +5
.TP
.B "play"
The play channel. If the file was onpened read-only
this value will be \fB<0\fR.
.TP
.B "record"
The recording channel. If the file was opened write-only
this value will be \fB<0\fR.
.RS -5

.TP
.B "SAM_IOC_CHANNEL_REDIRECT (IOR)"
Use an explicit channel number on this file descriptor.
The device driver will switch to that channel, if it's still available
or return an \fBEBUSY\fR error condition.
The channel numbers are given in in the structure \fBSamChannelNumbers\fR,
which is described with \fBSAM_IOC_CHANNEL_INFO\fR.
If any of the \fBplay\fR or \fBrecord\fR channel numbers are negative,
they will remain untouched. After a success redirection, the channel parameters
will be reset to their default values, just like after opening a fresh channel.

.TP
.B "SAM_IOC_AUDIO_GET (IOWR)"
Retrieve audio control values, that will be stored in the vector passed as:
.sp
.nf
typedef struct SamControl {
  unsigned char id, addr;
  char isConfigured;
  unsigned value;
} SamControl;
.fi
.sp
.RS +5
.TP
.B "id"
Defines the control id of the value you want to retrieve.
Which control id's are valid is determined by the \fBapiClass\fR
returned by \fBSAM_IOC_API_INFO\fR.
The last element in the vector needs to have an \fBid\fR field
with the value \fBSAM_AUDIO_ID_SENTINEL\fR to mark the end of
the vector.
.TP
.B "addr"
Some audio controls can be addressed on a left/right channel basis.
Use one of the constants \fBSAM_AUDIO_ADDR_LEFT\fR or \fBSAM_AUDIO_ADDR_RIGHT\fR
to address the right channel for this control.
.TP
.B "isConfigured"
If this audio control has been set by a previous call to
\fBSAM_IOC_AUDIO_SET\fR, isConfigured will contain a non-zero
value. Only the audio controls being configured are actually sent
to the firmware and only for those controls the \fBvalue\fR
field is valid.
However, on some firmwares, there exist many static audio controls,
that don't need to be configured after a channel opened
(e.g. Home Studio Recording).
Those audio controls will always be configured.
.TP
.B "value"
Will contain the value for this particular audio setting, but only
if the requested audio control is supported by the device and
the \fBisConfigured\fR field indicates that a value has been set before.
You can query the list of supported audio controls by using
\fBSAM_IOC_AUDIO_INFO\fR.
.RS -5

.TP
.B "SAM_IOC_AUDIO_SET (IOW)"
Set audio control values. The parameter being passed is described
in \fBSAM_IOC_AUDIO_GET\fR.
Each \fBvalue\fR for the audio controls being supported will be
sent to the firmware. The \fBisConfigured\fR field has no meaning
in this ioctl.

.TP
.B "SAM_IOC_AUDIO_INFO (IOWR)"
Query supported audio types. The parameter being passed is described
in \fBSAM_IOC_AUDIO_GET\fR. In case the given audio control is
supported by the firmware and/or hardware, the \fBvalue\fR field will
be set to a non-zero value. The \fBisConfigured\fR field has no meaning
in this ioctl.

.SH "IOCTL(/dev/sam9407/card\fR[0-9]\fB/sequencer\fR,\fB /dev/sam\fR[0-9]\fB_sequencer)"
.TP
.B "SAM_IOC_API_INFO (IOR)"
Refer to the \fBIOCTL(/dev/sam9407/card\fR[0-9]\fB/mixer\fR,\fB /dev/sam\fR[0-9]\fB_mixer)\fR
section for a description. Please note that on this device
the API version number returned shall be compared with the
constant \fBSAM_SEQ_API_VERSION\fR.
.TP
.B "SAM_IOC_SYNC_JOIN (IOWR)"
.TP
.B "SAM_IOC_SYNC_LEAVE (IO)"
.TP
.B "SAM_IOC_SYNC_INFO (IOR)"
Refer to the \fBIOCTL(/dev/sam9407/card\fR[0-9]\fB/mixer\fR,\fB /dev/sam\fR[0-9]\fB_mixer)\fR
section for a description.

.SH "IOCTL(/dev/sam9407/card\fR[0-9]\fB/mod\fR,\fB /dev/sam\fR[0-9]\fB_mod)"
This is the interface to the static wave buffer device.
Sound output is controlled by writing predefined chunks of data,
called events, to the device. Those events are queued
and will be played in the background, as soon as the defined wait timers,
that are a part of the event stream, expired.
For synchronisation purpose, an echo event can be used and can be
read back from the device, as soon as the playback engine
passed it's location.
Additionally, a number of ioctl's are available for non queued control
of the device.
.SS "MOD EVENTS"
Each event does have a structure element called \fBtype\fR at the
very beginning, that must be initialized with one of the constants
\fBSAM_MOD_EVENT_\fItype\fR.
In the list that follows, we list all event types along with the
structure type that shall be used for this event.
.TP
.B "SAM_MOD_EVENT_ECHO, SamModEchoEvent"
.sp
.nf
typedef struct {
  SamModEventType type;
  void *closure;
} SamModEchoEvent;
.fi
.sp
The event written to the mod device will be looped back, and is available
to \fBread(2)\fR as soon as the playback engine passed the position of this event.
The element \fBclosure\fR can take any value and the only purpose why it's there is to
give the user a way to distinguish his echo events.
.TP
.B "SAM_MOD_EVENT_WAIT_ABS, SamModTimerEvent"
.sp
.nf
typedef struct {
  SamModEventType type;
  struct timeval time;
} SamModTimerEvent;
.fi
.sp
Make the playback engine wait until the timestamp given in \fBtime\fR has passed.
The playback engine timer will start as soon as an \fBSAM_IOC_MOD_TIMER_START\fR
ioctl was issued, excluding periods of time when it's been stopped again by means
of the \fBSAM_IOC_MOD_TIMER_STOP\fR ioctl.
.TP
.B "SAM_MOD_EVENT_WAIT_REL, SamModTimerEvent"
Refer to \fBSAM_MOD_EVENT_WAIT_ABS\fR for a description of the
\fBSamModTimerEvent\fR structure.

Make the playback engine wait for the given amount of time.
The \fBtime\fR parameter defines a delta time from the last
\fBSAM_MOD_EVENT_WAIT_ABS\fR or \fBSAM_MOD_EVENT_WAIT_REL\fR call.
.TP
.B "SAM_MOD_EVENT_MEM, SamModMemEvent"
.sp
.nf
typedef struct {
  SamModEventType type;
  unsigned char voice, handle;
  SamModSampleType sampleType;
  SamModLoopType loopType;
  unsigned start, loop, end;
} SamModMemEvent;
.fi
.sp
Link an opened \fBvoice\fR to a sample, that has been uploaded
previously using the \fBSAM_IOC_MOD_SAMPLES_LOAD\fR ioctl.
.RS +5
.TP
.B "handle"
The sample identifier as passed to the \fBSAM_IOC_MOD_SAMPLES_LOAD\fR ioctl.
.TP
.B "sampleType"
This defines the format of the samples loaded with the \fBSAM_IOC_MOD_SAMPLES_LOAD\fR
ioctl.
.sp
.nf
typedef enum {
  SamModSample16Bit,
  SamModSample8BitLow,
  SamModSample8BitHigh,
} SamModSampleType;
.fi
.sp
8 bit samples need to be wrapped in 16 bit wide words,
with only the lower or high byte being relevant.
.TP
.B "loopType, start, loop, end"
All samples must loop, according to the \fBsam9407\fR firmware manual,
so you need to add 8 zero samples to the end of a non looping sample
and let it loop on those last 8 zero samples, that are not audible.
The looping type is given in \fBloopType\fR element,
which is defined as:
.sp
.nf
typedef enum {
  SamModLoopForward,
  SamModLoopReverse,
  SamModLoopReverseSignInvert,
} SamModLoopType;
.fi
.sp
Looping positions are given in the \fBstart\fR, \fBloop\fR and \fBend\fR
elements.
Playback starts at \fBstart\fR passes \fBloop\fR continues to \fBend\fR
and will then either jump back to \fBloop\fR without playback if \fBloopType\fR is
\fBSamModLoopForward\fR, or rewind to \fBloop\fR with playback in case
\fBloopType\fR is \fBSamModLoopReverse\fR or \fBSamModLoopReverseSignInvert\fR.
Once at \fBloop\fR it will continue to \fBend\fR and the whole cycle will
start again.
If the \fBSamModLoopReverseSignInvert\fR loop type is chosen,
the sample sign will be inverted whenever the playback direction changes.
.RS -5

.TP
.B "SAM_MOD_EVENT_OPEN, SamModTriggerEvent"
.sp
.nf
typedef struct {
  SamModEventType type;
  unsigned char voice;
} SamModTriggerEvent;
.fi
.sp
This will open the voice number passed in \fBvoice\fR. This is the very
first operation that needs to be done to access a voice.
.TP
.B "SAM_MOD_EVENT_CLOSE, SamModTriggerEvent"
Refer to \fBSAM_MOD_EVENT_OPEN\fR for a description of the
\fBSamModTriggerEvent\fR structure.

This will close the voice number passed in \fBvoice\fR.
.TP
.B "SAM_MOD_EVENT_START, SamModTriggerEvent"
Refer to \fBSAM_MOD_EVENT_OPEN\fR for a description of the
\fBSamModTriggerEvent\fR structure.

This will start playback of the voice number passed in \fBvoice\fR.
.TP
.B "SAM_MOD_EVENT_STOP, SamModTriggerEvent"
Refer to \fBSAM_MOD_EVENT_OPEN\fR for a description of the
\fBSamModTriggerEvent\fR structure.

This will stop playback of the voice number passed in \fBvoice\fR.
.TP
.B "SAM_MOD_EVENT_VOL, SamModVolOutEvent"
.sp
.nf
typedef struct {
  SamModEventType type;
  unsigned char voice;
  unsigned char volume;
} SamModVolOutEvent;
.fi
.sp
Sets the main audio output volume for the voice number passed
in \fBvoice\fR to \fBvolume\fR.
.TP
.B "SAM_MOD_EVENT_MAIN, SamModVolSendEvent"
.sp
.nf
typedef struct {
  SamModEventType type;
  unsigned char voice;
  unsigned char left, right;
} SamModVolSendEvent;
.fi
.sp
Sets the front audio output volume levels for the voice number passed
in \fBvoice\fR to \fBleft\fR and \fBright\fR
referring to the corresponding channels.
.TP
.B "SAM_MOD_EVENT_AUX, SamModVolSendEvent"
Refer to \fBSAM_MOD_EVENT_MAIN\fR for a description of the
\fBSamModVolSendEvent\fR structure.

Sets the rear audio output volume levels for the voice number passed
in \fBvoice\fR to \fBleft\fR and \fBright\fR
referring to the corresponding channels.
.TP
.B "SAM_MOD_EVENT_PITCH, SamModPitchEvent"
.sp
.nf
typedef struct {
  SamModEventType type;
  unsigned char voice;
  unsigned freq;
} SamModPitchEvent;
.fi
.sp
Sets the frequency for the voice number passed
in \fBvoice\fR to \fBfreq\fR.
.TP
.B "SAM_MOD_EVENT_FILT, SamModFilterEvent"
.sp
.nf
typedef struct {
  SamModEventType type;
  unsigned char voice;
  unsigned char q, fc;
} SamModFilterEvent;
.fi
.sp
Sets the resonance and cut frequency levels for the voice
passed in \fBvoice\fR.
.RS +5
.TP
.B "q"
This defines the resonance level. \fB255\fR disables
resonance and \fB0\fR turns on maximum resonance.
.TP
.B "fc"
This defines cut frequency level. \fB255\fR opens
the filter and \fB0\fR closes it.
.RS -5
.SS "MOD IOCTL's"
.TP
.B "SAM_IOC_API_INFO (IOR)"
Refer to the \fBIOCTL(/dev/sam9407/card\fR[0-9]\fB/mixer\fR,\fB /dev/sam\fR[0-9]\fB_mixer)\fR
section for a description. Please note that on this device
the API version number returned shall be compared with the
constant \fBSAM_MOD_API_VERSION\fR.
.TP
.B "SAM_IOC_SYNC_JOIN (IOWR)"
.TP
.B "SAM_IOC_SYNC_LEAVE (IO)"
.TP
.B "SAM_IOC_SYNC_INFO (IOR)"
Refer to the \fBIOCTL(/dev/sam9407/card\fR[0-9]\fB/mixer\fR,\fB /dev/sam\fR[0-9]\fB_mixer)\fR
section for a description.
.TP
.B "SAM_IOC_MOD_MEM_INFO (IOR)"
Return the soundcard memory information as:
.sp
.nf
typedef struct {
  unsigned memMod, memFree;
} SamModMemInfo;
.fi
.sp
.RS +5
.TP
.B "memMod"
The amount of memory used by static wave samples words (2 bytes).
.TP
.B "memFree"
The amount of free memory available on the card in words (2 bytes).
.RS -5
.TP
.B "SAM_IOC_MOD_RESET (IO)"
This will reset the device, and dispose all resources allocated with it.
The behaviour is similar to closing and reopening the device again.
.TP
.B "SAM_IOC_MOD_POST (IO)"
Wait until the queue of events pending at the playback engine is empty.
This can possibly take an infinite amount of time, if the timer wasn't
started.
.TP
.B "SAM_IOC_MOD_TIMER_START (IO)"
Start the playback engine timer.
.TP
.B "SAM_IOC_MOD_TIMER_STOP (IO)"
Stop the playback engine timer.
.TP
.B "SAM_IOC_MOD_SAMPLES_LOAD (IOW)"
Upload sound PCM samples to the soundcard defined in:
.sp
.nf
typedef struct {
  unsigned char handle;
  unsigned short *data;
  unsigned size;
} SamModSamples;
.fi
.sp
.RS +5
.TP
.B "handle"
A unique identifier to associate with these samples.
This identifier needs to be passed to \fBSAM_MOD_EVENT_MEM\fR events.
.TP
.B "data"
Wave PCM samples that shall be uploaded to the soundcard.
.TP
.B "size"
Number of wave PCM samples stored at \fBdata\fR.
.RS -5
.TP
.B "SAM_IOC_MOD_SAMPLES_UNLOAD (IOW)"
Unload sound PCM samples associated with the handle passed as:
.sp
.nf
unsigned char
.fi
.sp
.TP
.B "SAM_IOC_MOD_GET_VOICES (IOR)"
Query the number of voices available on the soundcard, that
will be returned as:
.sp
.nf
unsigned char
.fi
.sp
.TP
.B "SAM_IOC_MOD_GET_POS (IOWR)"
Return the current reading position of an opened voice in:
.sp
.nf
typedef struct {
  unsigned char voice;
  unsigned offset;
} SamModPosition;
.fi
.sp
.RS +5
.TP
.B "voice"
The voice we want to query the position for.
.TP
.B "offset"
The current position reading.
.RS -5
.TP
.B "SAM_IOC_MOD_ADD_POS (IOW)"
Add an offset to the current position reading of an opened voice,
defined in the structure of type \fBSamModPosition\fR, which is described
in \fBSAM_IOC_MOD_GET_POS\fR.
.TP
.B "SAM_IOC_MOD_CTRLRATE_INFO (IOR)"
Returns the timer resolution (in HZ) used by the driver.

.SH "MIXER EVENTS"
Whenever a mixer control value changes, a new firmware is uploaded,
or a soundbank entry changed, an event is sent to the user process,
that can be read(2) from the \fB/dev/sam9407/card\fR[0-9]\fB/mixer\fR,\fB /dev/sam\fR[0-9]\fB_mixer\fR device.
Each event read will be of type:
.sp
.nf
typedef struct {
  char type;
  unsigned char id, addr;
} SamEvent;
.fi
.sp
.TP
.B "type"
If \fB>=0\fR it will contain the audio channel as being returned by
a \fBSAM_IOC_CHANNEL_INFO\fR ioctl.
If \fBtype\fR is below zero, it will be one of the following constants:
.RS +5
.TP
.B "SAM_EVENT_TYPE_MIXER"
The \fBid\fR field contains a \fBSamMixerId\fR.
.TP
.B "SAM_EVENT_TYPE_NOCTRL"
The \fBid\fR field contains a \fBSamEventId\fR.
.RS -5
.TP
.B "id"
Contains either a \fBSamAudioId\fR, \fBSamMixerId\fR or
a \fBSamEventId\fR, depending on the value of the \fBtype\fR field.
\fBSamEventId\fR is defined as:
.sp
.nf
typedef enum {
  SAM_EVENT_FIRMWARE_CHANGED,
  SAM_EVENT_SBANK_CHANGED,
  SAM_EVENT_CHANNEL_OWNER_CHANGED,
} SamEventId;
.fi
.sp
.TP
.B "addr"
Contains the additional address tag of the control that changed.

.SH "SEE ALSO"
sam9407(8), sam9407.conf(5), x9407(1)

.SH AUTHOR
Gerd Rausch <gerd@meshed.net>
